/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the board support package (BSP) of a dedicated
*   target.
*   This template is responsible to initialize the display hardware of the board
*   and to provide the necessary access to update the display content.
*   The color format of the framebuffer has to correspond to the color format
*   of the Graphics Engine.
*
*   Important: This file is intended to be used as a template. Please adapt the
*   implementation according your particular hardware.
*
*******************************************************************************/

#include "dev_drv.h"
#include "rsk_switches.h"
#include "devdrv_intc.h"

#include "cache.h"
#include "GWP0700CNWV04.h"

#include "FreeRTOS.h"
#include "semphr.h"
#include "task.h"

#include <string.h>

#include "ewconfig.h"
#include "ewrte.h"

#include "ew_bsp_display.h"

static volatile int32_t       VSyncCount           = 0;
SemaphoreHandle_t             VSyncBinarySemaphore = NULL;

#define VDC5_INT_PRIORITY     (25u)

volatile uint8_t __attribute__ ((section (".VRAM_SECTION1"))) FrameBuffer_1[ EW_FRAME_BUFFER_SIZE ];
#define EW_FRAME_BUFFER_ADDR (unsigned long)FrameBuffer_1

#if EW_USE_DOUBLE_BUFFER == 1

  volatile uint8_t __attribute__ ((section (".VRAM_SECTION2"))) FrameBuffer_2[ EW_DOUBLE_BUFFER_SIZE ];
  #define EW_DOUBLE_BUFFER_ADDR (unsigned long)FrameBuffer_2

#else

  #error "Double buffering requested for RZ/A1H!"

#endif


/* helper function to register vdc5 interrupt */
static void DisplaySetInterrupt( vdc5_channel_t ch )
{
  void (*func)(uint32_t);
  uint16_t  int_id;
  int32_t   ret_code;

  int_id = ( ch == VDC5_CHANNEL_0 ) ? INTC_ID_GR3_VLINE0 : INTC_ID_GR3_VLINE1;

  func = R_VDC5_GetISR( ch, VDC5_INT_TYPE_VLINE );
  ret_code = R_INTC_RegistIntFunc( int_id, func );
  while (ret_code != DEVDRV_SUCCESS);

  ret_code = R_INTC_SetPriority( int_id, (uint8_t)VDC5_INT_PRIORITY );
  while (ret_code != DEVDRV_SUCCESS);

  ret_code = R_INTC_Enable( int_id );
  while (ret_code != DEVDRV_SUCCESS);
}


static void DisplayVsyncCallbackFunc( vdc5_int_type_t int_type )
{
  BaseType_t xHigherPriorityTaskWoken = pdFALSE;

  (void)int_type;

  if ( VSyncCount-- > 0)
  {
    if ( VSyncCount <=0 )
    {
      if ( VSyncBinarySemaphore != NULL)
        xSemaphoreGiveFromISR( VSyncBinarySemaphore, &xHigherPriorityTaskWoken );
    }
  }

  /* If xHigherPriorityTaskWoken was set to true we should yield.
     The actual macro used here is port specific. */
  portYIELD_FROM_ISR( xHigherPriorityTaskWoken );
}


static void DisplayWaitVsync( int32_t wait_count )
{
  VSyncCount = wait_count;
  while (VSyncCount > 0)
  {
    /* let the task sleep until it gets notified by the ISR */
    xSemaphoreTake( VSyncBinarySemaphore, portMAX_DELAY);
  }
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayInit
*
* DESCRIPTION:
*   The function EwBspDisplayInit initializes the display hardware and returns
*   the display parameter.
*
* ARGUMENTS:
*   aGuiWidth,
*   aGuiHeight   - Size of the GUI in pixel.
*   aDisplayInfo - Display info data structure.
*
* RETURN VALUE:
*   Returns 1 if successful, 0 otherwise.
*
*******************************************************************************/
int EwBspDisplayInit( int aGuiWidth, int aGuiHeight, XDisplayInfo* aDisplayInfo )
{
  vdc5_error_t     error;
  vdc5_gr_format_t vdc5_gr_format = VDC5_GR_FORMAT_ARGB8888;
  vdc5_int_t       vdc5_int;

  EW_UNUSED_ARG( aGuiWidth );
  EW_UNUSED_ARG( aGuiHeight );

  /* check and clean display info structure */
  if ( !aDisplayInfo )
    return 0;
  memset( aDisplayInfo, 0, sizeof( XDisplayInfo ));

  /* pixel format configuration: translate given framebuffer color format into VDC5 mode */
  #if ( EW_FRAME_BUFFER_COLOR_FORMAT == EW_FRAME_BUFFER_COLOR_FORMAT_RGBA8888 )
    vdc5_gr_format = VDC5_GR_FORMAT_ARGB8888;
  #elif ( EW_FRAME_BUFFER_COLOR_FORMAT == EW_FRAME_BUFFER_COLOR_FORMAT_RGB565 )
    vdc5_gr_format = VDC5_GR_FORMAT_RGB565;
  #else
    #error The given EW_FRAME_BUFFER_COLOR_FORMAT is not supported! Check your project settings!
  #endif

  error = InitVDC5( (void*)EW_FRAME_BUFFER_ADDR, (uint16_t)EW_FRAME_BUFFER_WIDTH, (uint16_t)EW_FRAME_BUFFER_HEIGHT, vdc5_gr_format );
  if ( error != VDC5_OK )
    EwPrint( "EwBspDisplayInit: Failed to initialize VDC5...\n" );

  VSyncBinarySemaphore = xSemaphoreCreateBinary();
  if ( VSyncBinarySemaphore == NULL )
	  EwPrint( "EwBspDisplayInit: Failed to initialize VSyncBinarySemaphore...\n" );

  DisplaySetInterrupt( VDC5_CHANNEL_0 );

  vdc5_int.type = VDC5_INT_TYPE_VLINE;
  vdc5_int.line_num = 0;
  vdc5_int.callback = &DisplayVsyncCallbackFunc;

  error = R_VDC5_CallbackISR( VDC5_CHANNEL_0, &vdc5_int );
  if ( error != VDC5_OK )
    EwPrint( "EwBspDisplayInit: Failed to register V-sync callback...\n" );

  /* return the current display configuration */
  aDisplayInfo->FrameBuffer    = (void*)EW_FRAME_BUFFER_ADDR;
  aDisplayInfo->DoubleBuffer   = (void*)EW_DOUBLE_BUFFER_ADDR;
  aDisplayInfo->BufferWidth    = EW_FRAME_BUFFER_WIDTH;
  aDisplayInfo->BufferHeight   = EW_FRAME_BUFFER_HEIGHT;
  aDisplayInfo->DisplayWidth   = EW_DISPLAY_WIDTH;
  aDisplayInfo->DisplayHeight  = EW_DISPLAY_HEIGHT;

  #if EW_USE_DOUBLE_BUFFER == 1
    aDisplayInfo->UpdateMode   = EW_BSP_DISPLAY_UPDATE_NORMAL;
  #else
    aDisplayInfo->UpdateMode   = EW_BSP_DISPLAY_UPDATE_PARTIAL;
  #endif

  return 1;
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayDone
*
* DESCRIPTION:
*   The function EwBspDisplayDone deinitializes the display hardware.
*
* ARGUMENTS:
*   aDisplayInfo - Display info data structure.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspDisplayDone( XDisplayInfo* aDisplayInfo )
{
  EW_UNUSED_ARG( aDisplayInfo );
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayGetUpdateArea
*
* DESCRIPTION:
*   The function EwBspDisplayGetUpdateArea returns the next update area
*   depending on the selected display mode:
*   In case of a synchroneous single-buffer, the function has to return the
*   the rectangular areas that correspond to the horizontal stripes (fields)
*   of the framebuffer.
*   In case of a scratch-pad buffer, the function has to return the subareas
*   that fit into the provided update rectangle.
*   During each display update, this function is called until it returns 0.
*
* ARGUMENTS:
*   aUpdateRect - Rectangular area which should be updated (redrawn).
*
* RETURN VALUE:
*   Returns 1 if a further update area can be provided, 0 otherwise.
*
*******************************************************************************/
int EwBspDisplayGetUpdateArea( XRect* aUpdateRect )
{
  return 0;
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayWaitForCompletion
*
* DESCRIPTION:
*   The function EwBspDisplayWaitForCompletion is called from the Graphics Engine
*   to ensure that all pending activities of the display system are completed, so
*   that the rendering of the next frame can start.
*   In case of a double-buffering system, the function has to wait until the
*   V-sync has occured and the pending buffer is used by the display controller.
*   In case of an external display controller, the function has to wait until
*   the transfer (update) of the graphics data has been completed and there are
*   no pending buffers.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspDisplayWaitForCompletion( void )
{
  #if EW_USE_DOUBLE_BUFFER == 1

    DisplayWaitVsync( 1 );

  #endif
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplayCommitBuffer
*
* DESCRIPTION:
*   The function EwBspDisplayCommitBuffer is called from the Graphics Engine
*   when the rendering of a certain buffer has been completed.
*   The type of buffer depends on the selected framebuffer concept.
*   If the display is running in a double-buffering mode, the function is called
*   after each buffer update in order to change the currently active framebuffer
*   address. Changing the framebuffer address should be synchronized with V-sync.
*   If the system is using an external graphics controller, this function is
*   responsible to start the transfer of the framebuffer content.
*
* ARGUMENTS:
*   aAddress - Address of the framebuffer to be shown on the display.
*   aX,
*   aY       - Origin of the area which has been updated by the Graphics Engine.
*   aWidth,
*   aHeight  - Size of the area which has been updated by the Graphics Engine.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspDisplayCommitBuffer( void* aAddress, int aX, int aY, int aWidth, int aHeight )
{
  static vdc5_read_chg_t  read_chg;

  /* flush framebuffer area from cache to memory */
  L1_D_CacheCleanRange( (uint32_t)aAddress, (uint32_t)aAddress + EW_FRAME_BUFFER_SIZE );

  /* framebuffer base address */
  read_chg.gr_base       = (void*)aAddress;
  read_chg.width_read_fb = NULL;
  read_chg.gr_grc        = NULL;
  read_chg.gr_disp_sel   = NULL;

  if ( R_VDC5_ChangeReadProcess(VDC5_CHANNEL_0, VDC5_LAYER_ID_2_RD, &read_chg) != VDC5_OK )
    EwPrint( "EwBspDisplayCommitBuffer(): Failed to change framebuffer address!\n" );
}


/*******************************************************************************
* FUNCTION:
*   EwBspDisplaySetClut
*
* DESCRIPTION:
*   The function EwBspDisplaySetClut is called from the Graphics Engine
*   in order to update the hardware CLUT of the current framebuffer.
*   The function is only called when the color format of the framebuffer is
*   Index8 or LumA44.
*
* ARGUMENTS:
*   aClut - Pointer to a color lookup table with 256 entries.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspDisplaySetClut( unsigned long* aClut )
{
}


/* msy */
