/******************************************************************************
* Copyright(c) 2010-2012 Renesas Electronics Corporation. All rights reserved.
*
* brief  : NCG State variable control functions.
*
* author : Renesas Electronics Corporation
*
* history: 2010.10.08
*          - Created the initial code.
*          2016.01.19
*          - Applied to OSPL and RGA.
*
* - $ModuleRevision: {Masters}\RGPNCG\RZ\24 $
* - $Rev: $
* - $Date: 2017-09-25T14:07:01+09:00 $
*******************************************************************************/

/*=============================================================================
 * Includes
 */
#include "RGA_Config.h"
#include "r_ospl.h"

#include    "ncg_defs.h"
#include    "ncg_debug.h"
#include    "ncg_state.h"
#include    "ncg_vg_isr.h"

#define  GS_FOR_OSPL  1     /* 1 or 0 */
#define  GS_STATE_WATCH  0  /* 1 or 0 */
#ifdef  RGAH_VERSION

/*=============================================================================
 * Internal definitions
 */

#define     NCG_MAX_STATE_OBJECTS   (1)

struct T_NCG_STATE
{
    NCGint32        stid;
    NCGint32        bInUse;
    NCGuint32       ui32State;

#if  GS_FOR_OSPL
    r_ospl_thread_id_t  ReceiverThreadID;
    bit_flags32_t       EventValueForOSPL;
#endif
} ;


/*=============================================================================
 *  Prototyping of internal functions
 */

static NCGint32 ncg_init_states( void ) ;


/*=============================================================================
 *  Private global variables and functions
 */

static volatile struct T_NCG_STATE  tStates[NCG_MAX_STATE_OBJECTS];

static NCGint32 ncg_state_initialized = 0 ;
/* NCGboolean */

static uint32_t                      gs_next_event_value = 0;
static volatile struct T_NCG_STATE*  gs_ncg_last_created_state;


/*=============================================================================
 *  Global Function
 */

/*----------------------------------------------------------------------------
 NAME       : NCGSYS_CreateState
 FUNCTION   : Create a state control object.
 PARAMETERS : ppObj       : [OUT] The pointer to the created state control object.
              ui32StateID : [IN ] The identifier of the state variable.
 RETURN     : Error code of the NCG.
------------------------------------------------------------------------------*/
NCGint32
NCGSYS_CreateState(
    NCGvoid         **ppObj,
    NCGuint32       ui32StateID )
{
    NCGint32 ret ;
    NCGint32 i ;
    volatile struct T_NCG_STATE  *pState = NCG_NULL;
    errnum_t  e;

    NCG_DEBUG_PRINT_STRING( "[LOG] NCGSYS_CreateState" );

    NCG_UNREFERENCED_PARAMETER(ui32StateID);

    if( ppObj == NCG_NULL )
    {
        NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_CreateState(1) invalid ppObj.%s" )
        NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
        NCG_DEBUG_MAKE_MSG_END();
        NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
        return NCG_err_unknown ;
    }

    if( ncg_state_initialized == 0 )
    {
        ret = ncg_init_states() ;
        if( ret != NCG_no_err )
        {
            NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_CreateState(2) falsed:ncg_init_states(%d).%s" )
            NCG_DEBUG_MAKE_MSG_PARAMETER( ret )
            NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
            NCG_DEBUG_MAKE_MSG_END();
            NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
            return ret ;
        }
        ncg_state_initialized = 1 ;
    }

    for( i = 0 ; i < NCG_MAX_STATE_OBJECTS ; i++ )
    {
        if( tStates[ i ].bInUse == 0 )
        {
            tStates[ i ].bInUse = 1 ;
            pState = (volatile struct T_NCG_STATE*)&tStates[ i ] ;
            break ;
        }
    }
    if( pState == NCG_NULL )
    {
        NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_CreateState(3) Not find.%s" )
        NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
        NCG_DEBUG_MAKE_MSG_END();
        NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
        return NCG_err_resource_exceeded ;
    }

#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wcast-qual"
#endif

    pState->ui32State = 0U ;
    *ppObj = ( NCGvoid *)pState ;

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif

#if  GS_FOR_OSPL
    if ( gs_next_event_value != 0 )
    {
        bit_flags32_t  event_value;

        e= R_OSPL_EVENT_Allocate(
               &pState->ReceiverThreadID,  R_OSPL_THREAD_GetCurrentId(),
               &event_value,  gs_next_event_value );
        gs_next_event_value = 0;
        IF(e)
        {
            return NCG_err_resource_exceeded ;
        }
        pState->EventValueForOSPL = event_value;
    }
    gs_ncg_last_created_state = pState;
#endif

    return NCG_no_err ;
}

/*----------------------------------------------------------------------------
 NAME       : NCGSYS_DestroyState
 FUNCTION   : Destroy the given state control object.
 PARAMETERS : pObj : [IN ] The pointer to the state control object.
 RETURN     : Error code of the NCG.
------------------------------------------------------------------------------*/
NCGint32
NCGSYS_DestroyState(
    NCGvoid *pObj )
{
    struct T_NCG_STATE *pState = ( struct T_NCG_STATE *)pObj ;
    NCGint32 stid = pState->stid ;
    NCGint32 i ;
    NCGboolean bDestroyFlg ;
    errnum_t   e;

    NCG_DEBUG_PRINT_STRING( "[LOG] NCGSYS_DestroyState" );

    if( pState == NCG_NULL )
    {
        NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_DestroyState(1) Failed: pState value is NULL.%s" )
        NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
        NCG_DEBUG_MAKE_MSG_END();
        NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
        return NCG_err_unknown ;
    }
    if( &tStates[ stid ] != pState )
    {
        NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_DestroyState(2) Failed: Invalid object.%s" )
        NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
        NCG_DEBUG_MAKE_MSG_END();
        NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
        return NCG_err_unknown ;
    }
    if( pState->bInUse == 0 )
    {
        NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_DestroyState(3) Failed: Object is not used.%s" )
        NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
        NCG_DEBUG_MAKE_MSG_END();
        NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
        return NCG_err_unknown ;
    }
    tStates[ stid ].bInUse = 0 ;
    tStates[ stid ].ui32State = 0U ;

#if  GS_FOR_OSPL
    e= R_OSPL_EVENT_Free( &pState->ReceiverThreadID,  &pState->EventValueForOSPL );
    IF(e)
    {
        R_NOOP();
    }
    pState->EventValueForOSPL = 0u;
#endif

    bDestroyFlg = NCG_TRUE ;
    for( i = 0 ; i < NCG_MAX_STATE_OBJECTS ; i++ )
    {
        if( tStates[ i ].bInUse != 0 )
        {
            bDestroyFlg = NCG_FALSE ;
            break ;
        }
    }
    if( bDestroyFlg != NCG_FALSE )
    {
        ncg_state_initialized = 0 ;
    }
    return NCG_no_err ;
}

/*----------------------------------------------------------------------------
 NAME       : NCGSYS_SetState
 FUNCTION   : Set the state control object into the given state.
 PARAMETERS : pObj      : [IN ] The pointer to the state control object.
              ui32State : [IN ] The state value to set.
              ui32Flags : [IN ] Flags.
 RETURN     : Error code of the NCG.
------------------------------------------------------------------------------*/
NCGint32
NCGSYS_SetState(
    NCGvoid         *pObj,
    NCGuint32       ui32State,
    NCGuint32       ui32Flags )
{
    volatile struct T_NCG_STATE *pState = ( volatile struct T_NCG_STATE *)pObj ;

    NCG_DEBUG_PRINT_STRING( "[LOG] NCGSYS_SetState" );

    NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "( 0x%08X, %X )%s" )
    NCG_DEBUG_MAKE_MSG_PARAMETER( ui32State )
    NCG_DEBUG_MAKE_MSG_PARAMETER( ui32Flags )
    NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
    NCG_DEBUG_MAKE_MSG_END();
    NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );

    NCGVG_DisableInterrupt();

#if  GS_STATE_WATCH
    if ( ( ui32Flags & NCGSYS_STATE_SET_MASK ) == NCGSYS_STATE_SET_AND )
    {
        R_D_AddToIntLog( 0x60C00000 );
        R_D_AddToIntLog( (int) pObj );
        R_D_AddToIntLog( (int) R_OSPL_THREAD_GetCurrentId() );
        R_D_AddToIntLog( ~ui32State );
    }
    else if ( ( ui32Flags & NCGSYS_STATE_SET_MASK ) == NCGSYS_STATE_SET_OR )
    {
        R_D_AddToIntLog( 0x60200000 );
        R_D_AddToIntLog( (int) pObj );
        R_D_AddToIntLog( (int) R_OSPL_THREAD_GetCurrentId() );
        R_D_AddToIntLog( ui32State );
    }
    else if ( ( ui32Flags & NCGSYS_STATE_SET_MASK ) == NCGSYS_STATE_SET_SET )
    {
        R_D_AddToIntLog( 0x60A00000 );
        R_D_AddToIntLog( (int) pObj );
        R_D_AddToIntLog( (int) R_OSPL_THREAD_GetCurrentId() );
        R_D_AddToIntLog( ui32State );
    }
#endif

    if( pState == NCG_NULL )
    {
        NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_SetState(1) Failed:pObj is NULL.%s" )
        NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
        NCG_DEBUG_MAKE_MSG_END();
        NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
        return NCG_err_unknown ;
    }
    if( &tStates[ pState->stid ] != pState )
    {
        NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_SetState(2) Failed:status (0x%08x).%s" )
        NCG_DEBUG_MAKE_MSG_PARAMETER( pState->stid )
        NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
        NCG_DEBUG_MAKE_MSG_END();
        NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
        return NCG_err_unknown ;
    }
    if( pState->bInUse == 0 )
    {
        NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_SetState(3) Failed: pObj is not used.%s" )
        NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
        NCG_DEBUG_MAKE_MSG_END();
        NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
        return NCG_err_unknown ;
    }

    switch( ui32Flags & NCGSYS_STATE_SET_MASK )
    {
    case NCGSYS_STATE_SET_AND :
        pState->ui32State &= ui32State ;
        break ;

    case NCGSYS_STATE_SET_OR :
        pState->ui32State |= ui32State ;
        break ;

    case NCGSYS_STATE_SET_SET :
        pState->ui32State = ui32State ;
        break ;

    default :
        NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_SetState(5) Failed:ui32Flags (0x%08x).%s" )
        NCG_DEBUG_MAKE_MSG_PARAMETER( ui32Flags )
        NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
        NCG_DEBUG_MAKE_MSG_END();
        NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
        return NCG_err_bad_parameter ;
    }

#if  GS_STATE_WATCH
    R_D_AddToIntLog( 0x60F00000 );
    R_D_AddToIntLog( pState->ui32State );
#endif

    NCGVG_EnableInterrupt();

#if  GS_FOR_OSPL
    {
        r_ospl_thread_id_t  thread;

        ASSERT_R( pState->EventValueForOSPL != 0,  return NCG_err_bad_parameter );

        thread = pState->ReceiverThreadID;  /* ICCARM Pa082 : volatile's order */
        R_OSPL_EVENT_Set( thread,  pState->EventValueForOSPL );
        /* Call R_OSPL_EVENT_Set only as notification of modification. */
        /* It is necessary to T-Lock exclusive control "pState->ui32State" and "pState->EventValueForOSPL", */
        /* when "pState->ui32State == 0" and "R_OSPL_EVENT_Clear" was called. */
        /* But OSPL does not have T-Lock API. The reason of no API is that it can operate by the above code. */
    }
#endif

    return NCG_no_err ;
}

/*----------------------------------------------------------------------------
 NAME       : NCGSYS_GetState
 FUNCTION   : Get the current state value from the state control object.
 PARAMETERS : pObj       : [IN ] The pointer to the state control object.
              ui32Flags  : [IN ] Flags.
 RETURN     : Error code of the NCG.
------------------------------------------------------------------------------*/
NCGuint32
NCGSYS_GetState(
    NCGvoid         *pObj,
    NCGuint32       ui32Flags )
{
    volatile struct T_NCG_STATE *pState = ( volatile struct T_NCG_STATE *)pObj ;

    R_UNREFERENCED_VARIABLE( ui32Flags );

    NCG_DEBUG_PRINT_STRING( "[LOG] NCGSYS_GetState" );

    if( pState == NCG_NULL )
    {
        NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_GetState(1) Failed: pObj.%s" )
        NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
        NCG_DEBUG_MAKE_MSG_END();
        NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
        return (NCGuint32)NCG_err_unknown ;
    }
    if( &tStates[ pState->stid ] != pState )
    {
        NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_GetState(2) Failed: pObj status(%d).%s" )
        NCG_DEBUG_MAKE_MSG_PARAMETER( pState->stid )
        NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
        NCG_DEBUG_MAKE_MSG_END();
        NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
        return (NCGuint32)NCG_err_unknown ;
    }
    if( pState->bInUse == 0 )
    {
        NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_GetState(3) Failed: pObj is not used.%s" )
        NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
        NCG_DEBUG_MAKE_MSG_END();
        NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
        return (NCGuint32)NCG_err_unknown ;
    }
    return ( NCGuint32 )pState->ui32State ;
}

/*----------------------------------------------------------------------------
 NAME       : NCGSYS_WaitState
 FUNCTION   : Wait for the state control object to be in the given state.
 PARAMETERS : pObj        : [IN ] The pointer to the state control object.
              ui32State   : [IN ] The state value to wait for.
              ui32Flags   : [IN ] Flags.
              ui32Timeout : [IN ] Timeout value in milli seconds.
 RETURN     : Error code of the NCG.
 DESCRIPTION: State is not cleared.
              When 2 bits was set, this function can be called
              to wait for one bit after calling this function
              to wait for other bit.
              This function can be called many times for a bit after the bit
              was set 1 time.
------------------------------------------------------------------------------*/
NCGint32
NCGSYS_WaitState(
    NCGvoid         *pObj,
    NCGuint32       ui32State,
    NCGuint32       ui32Flags,
    NCGuint32       ui32Timeout )
{
    volatile struct T_NCG_STATE *pState = ( volatile struct T_NCG_STATE *)pObj ;
    NCGint32 stid = pState->stid ;
    enum { NCGWAIT_OR, NCGWAIT_AND } wait_mode ;
    uint32_t   got_flags;
    errnum_t   e;

    NCG_DEBUG_PRINT_STRING( "[LOG] NCGSYS_WaitState" );

    NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "( 0x%08X, %X )%s" )
    NCG_DEBUG_MAKE_MSG_PARAMETER( ui32State )
    NCG_DEBUG_MAKE_MSG_PARAMETER( ui32Flags )
    NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
    NCG_DEBUG_MAKE_MSG_END();
    NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );

#if  GS_STATE_WATCH
    R_D_AddToIntLog( 0x60BE0000 );
    R_D_AddToIntLog( (int) pObj );
    R_D_AddToIntLog( (int) R_OSPL_THREAD_GetCurrentId() );
    R_D_AddToIntLog( ui32State );
    R_D_AddToIntLog( ui32Flags );
#endif

    wait_mode = (( ui32Flags & NCGSYS_STATE_WAIT_MASK ) == NCGSYS_STATE_WAIT_OR ) ? NCGWAIT_OR : NCGWAIT_AND ;

    if( pState == NCG_NULL )
    {
        NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_WaitState(1) Failed: pObj is NULL.%s" )
        NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
        NCG_DEBUG_MAKE_MSG_END();
        NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
        return NCG_err_unknown ;
    }
    if( &tStates[ stid ] != pState )
    {
        NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_WaitState(2) Failed: pObj.%s" )
        NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
        NCG_DEBUG_MAKE_MSG_END();
        NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
        return NCG_err_unknown ;
    }
    if( pState->bInUse == 0 )
    {
        NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_WaitState(3) Failed: pObj.%s" )
        NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
        NCG_DEBUG_MAKE_MSG_END();
        NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
        return NCG_err_unknown ;
    }
    if(( ui32Timeout != NCG_TIMEOUT_INFINITE ) && ( ui32Timeout > 0x7fffffffU ))
    {
        NCG_DEBUG_MAKE_MSG_START( NCG_G_MASSAGE_BUFF, "[NCG][ERROR]NCGSYS_WaitState(4) Failed: pObj.%s" )
        NCG_DEBUG_MAKE_MSG_PARAMETER( NCG_CRLF )
        NCG_DEBUG_MAKE_MSG_END();
        NCG_DEBUG_PRINT_MSG( NCG_G_MASSAGE_BUFF );
        return NCG_err_unknown ;
    }
#if  GS_FOR_OSPL
    ASSERT_R( pState->EventValueForOSPL != 0,  return NCG_err_bad_parameter );
#endif

    if( ui32Timeout != NCG_TIMEOUT_INFINITE )
    {
#if  GS_FOR_OSPL
        e = R_OSPL_EVENT_Wait( pState->EventValueForOSPL, &got_flags, ui32Timeout );
        IF ( e != 0 )
        {
            return NCG_err_unknown ;
        }
#else
        NCGint32 i32SignedTimeout = ( NCGint32 )ui32Timeout ;
        volatile NCGuint32 count ;
#endif

        while((( wait_mode == NCGWAIT_AND ) && ( pState->ui32State & ui32State ) != ui32State ) ||
                (( wait_mode == NCGWAIT_OR ) && ( pState->ui32State & ui32State ) == 0 ))
        {

#if  GS_FOR_OSPL
            e = R_OSPL_EVENT_Wait( pState->EventValueForOSPL, &got_flags, ui32Timeout );
            IF ( e != 0 )
            {
                return NCG_err_unknown ;
            }
#else
            NCGuint32 time ;

            time = NCGSYS_GetTickCount() ;

            i32SignedTimeout -= time ;
            if( i32SignedTimeout < 0 )
            {
                return NCG_err_wait_timeout ;
            }
#endif
        }
    }
    else
    {
#if  GS_FOR_OSPL
        e = R_OSPL_EVENT_Wait( pState->EventValueForOSPL, &got_flags, ui32Timeout );
        IF ( e != 0 )
        {
            return NCG_err_unknown ;
        }
#endif

        while((( wait_mode == NCGWAIT_AND ) && ( pState->ui32State & ui32State ) != ui32State ) ||
                (( wait_mode == NCGWAIT_OR ) && ( pState->ui32State & ui32State ) == 0 ))
        {

#if  GS_FOR_OSPL
            e = R_OSPL_EVENT_Wait( pState->EventValueForOSPL, &got_flags, ui32Timeout );
            IF ( e != 0 )
            {
                return NCG_err_unknown ;
            }
#endif
        }
    }

#if  GS_FOR_OSPL
    R_OSPL_EVENT_Set( R_OSPL_THREAD_GetCurrentId(), pState->EventValueForOSPL );
    /* For that any target bit was already set in next calling to wait for the bit. */
#endif

#if  GS_STATE_WATCH
    R_D_AddToIntLog( 0x60AF0000 );
#endif

    return NCG_no_err ;
}


/*=============================================================================
 *  Internal functions
 */

/*----------------------------------------------------------------------------
 NAME       : ncg_init_states
 FUNCTION   : Initialize global variable in NCG state.
 PARAMETERS : None.
 RETURN     : Error code of the NCG.
------------------------------------------------------------------------------*/
static NCGint32
ncg_init_states( void )
{
    NCGint32 i ;

    for( i = 0 ; i < NCG_MAX_STATE_OBJECTS ; i++ )
    {
        tStates[ i ].stid = i ;
        tStates[ i ].bInUse = 0 ;
        tStates[ i ].ui32State = 0U ;

#if  GS_FOR_OSPL
        tStates[ i ].EventValueForOSPL = 0;
        tStates[ i ].ReceiverThreadID = R_OSPL_THREAD_NULL;
#endif
    }
    return NCG_no_err;
}

/*----------------------------------------------------------------------------
 NAME       : NCGSYS_SetNextStateEventValue
 FUNCTION   : Set the value of thread attached event for next state.
 PARAMETERS : ui32EventValue : [IN ] The value of thread attached event.
 RETURN     : None.
------------------------------------------------------------------------------*/
#if  GS_FOR_OSPL
NCGvoid
NCGSYS_SetNextStateEventValue(
    NCGuint32       ui32EventValue )
{
    NCG_DEBUG_PRINT_STRING( "[LOG] NCGSYS_SetNextStateEventValue" );

    gs_next_event_value = ui32EventValue;
}
#endif  /* GS_FOR_OSPL */

/*----------------------------------------------------------------------------
 NAME       : NCGSYS_GetLastCreatedState
 FUNCTION   : GetLastCreatedState.
 PARAMETERS : None.
 RETURN     : LastCreatedState.
------------------------------------------------------------------------------*/
#if  GS_FOR_OSPL
NCGvoid*
NCGSYS_GetLastCreatedState(void)
{
    NCG_DEBUG_PRINT_STRING( "[LOG] NCGSYS_GetLastCreatedState" );

#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wcast-qual"
#endif

    return  (NCGvoid*) gs_ncg_last_created_state;

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif
}
#endif  /* GS_FOR_OSPL */

/*----------------------------------------------------------------------------
 NAME       : NCGSYS_SetStateEventValue
 FUNCTION   : Set the value of thread attached event.
 PARAMETERS : pObj           : [IN ] The pointer to the state control object.
              ui32EventValue : [IN ] The value of thread attached event.
 RETURN     : None.
------------------------------------------------------------------------------*/
#if  GS_FOR_OSPL
NCGvoid
NCGSYS_SetStateEventValue(
    NCGvoid         *pObj,
    NCGuint32       ui32EventValue )
{
    volatile struct T_NCG_STATE *pState = (volatile struct T_NCG_STATE *)pObj ;
    errnum_t  e;
    uint32_t  event_value;

    r_ospl_thread_id_t  thread;


    NCG_DEBUG_PRINT_STRING( "[LOG] NCGSYS_SetStateEventValue" );

    if ( ui32EventValue != 0 )
    {
        e= R_OSPL_EVENT_Allocate(
               &thread,  R_OSPL_THREAD_GetCurrentId(),
               &event_value,  ui32EventValue );
        IF(e)
        {
            R_OSPL_RaiseUnrecoverable( e );
        }

        pState->ReceiverThreadID = thread;
        pState->EventValueForOSPL = event_value;

        R_OSPL_EVENT_Set( thread,  event_value );
        /* For caling "NCGSYS_WaitState" immediately for "pState->ui32State" that was set */
        /* before JPEG decoding using same "event_value". */
    }
    else
    {
        e= R_OSPL_EVENT_Free( &pState->ReceiverThreadID,  &pState->EventValueForOSPL );
        IF(e)
        {
            R_OSPL_RaiseUnrecoverable( e );
        }
        pState->EventValueForOSPL = 0;
    }
}
#endif  /* GS_FOR_OSPL */

#endif  /* RGAH_VERSION */
/* -- end of file -- */
